@@ -77,6 +77,7 @@ sub edit {
     $form->{vc} = 'customer';
   }
 
+  $form->{shipto} = 1;
   &order_links;
   &prepare_order;
   &display_form;
@@ -130,14 +131,13 @@ sub order_links {
     }
   }
   
-  for (qw(terms taxincluded)) { $temp{$_} = $form->{$_} }
-  $form->{shipto} = 1 if $form->{id};
+  for (qw(terms taxincluded intnotes)) { $temp{$_} = $form->{$_} }
 
   # get customer / vendor
   AA->get_name(\%myconfig, \%$form);
-
+  
   if ($form->{id}) {
-    for (qw(terms taxincluded)) { $form->{$_} = $temp{$_} }
+    for (keys %temp) { $form->{$_} = $temp{$_} }
   }
 
   $form->{terms} = "" if ! $form->{terms};
@@ -298,6 +298,21 @@ sub prepare_order {
 }
 
 
+sub lookup_order {
+
+  $form->isblank("ordnumber", $locale->text('Order Number missing!'));
+
+  if ($id = OE->lookup_order(\%myconfig, \%$form)) {
+    $type = ($form->{vc} eq 'customer') ? 'sales_order' : 'purchase_order';
+    $form->{callback} = "oe.pl?action=edit&id=$id&type=$type&vc=$form->{vc}&path=$form->{path}&login=$form->{login}";
+    $form->redirect;
+  } else {
+    $form->error($locale->text('Order not on file!'));
+  }
+
+}
+
+
 sub form_header {
 
   $checkedopen = ($form->{closed}) ? "" : "checked";
@@ -374,7 +389,7 @@ sub form_header {
 		<td>
 		  <table>
 		    <tr>
-		      <td colspan=4>$form->{city}, $form->{state} $form->{country}</td>
+		      <td colspan=4>$form->{city} $form->{state} $form->{country}</td>
 		    </tr>
 		  </table>
 		</td>
@@ -410,7 +425,7 @@ sub form_header {
 		<td>
 		  <table>
 		    <tr>
-		      <td colspan=4>$form->{city}, $form->{state} $form->{country}</td>
+		      <td colspan=4>$form->{city} $form->{state} $form->{country}</td>
 		    </tr>
 		    <tr>
 		      <th align=right nowrap>|.$locale->text('Credit Limit').qq|</th>
@@ -1099,6 +1114,13 @@ sub search {
     $ordlabel = $locale->text('Order Number');
     $ordnumber = 'ordnumber';
     $employee = $locale->text('Salesperson');
+    $detail = qq|
+                <tr>
+		  <td><input name=detail type=radio class=radio value=0 checked> |.$locale->text('Summary').qq|</td>
+		  <td><input name=detail type=radio class=radio value=1> |.$locale->text('Detail').qq|
+		  </td>
+		</tr>
+|;
   }
   
   if ($form->{type} eq 'consolidate_purchase_order') {
@@ -1344,6 +1366,7 @@ sub search {
           <td colspan=3>
 	    <table>
 	      $openclosed
+	      $detail
 |;
 
   while (@a) {
@@ -1540,6 +1563,7 @@ sub transactions {
   unshift @columns, "runningnumber";
 
   $form->{l_open} = $form->{l_closed} = "Y" if ($form->{open} && $form->{closed}) ;
+  $form->{l_memo} = "Y" if $form->{detail};
 
   for (@columns) {
     if ($form->{"l_$_"} eq "Y") {
@@ -1630,6 +1654,7 @@ sub transactions {
 	$button{'Order Entry--Purchase Order'}{code} = qq|<input class=submit type=submit name=action value="|.$locale->text('Generate Purchase Orders').qq|"> |;
 	$button{'Order Entry--Purchase Order'}{order} = $i++;
       }
+      $callback .= "&detail=$form->{detail}";
     }
     if ($form->{type} eq 'consolidate_sales_order') {
       $form->{title} = $locale->text('Sales Orders');
@@ -1800,7 +1825,9 @@ function CheckAll() {
     $column_data{reqdate} = "<td nowrap>$ref->{reqdate}&nbsp;</td>";
 
     $column_data{runningnumber} = qq|<td align=right>$i</td>|;
-    $column_data{ndx} = qq|<td><input name="ndx_$i" class=checkbox type=checkbox value=$ref->{id} checked></td>|;
+    $id = ($ref->{orderitemsid}) ? "$ref->{id}--$ref->{orderitemsid}" : $ref->{id};
+    
+    $column_data{ndx} = qq|<td><input name="ndx_$i" class=checkbox type=checkbox value=$id checked></td>|;
     $column_data{$ordnumber} = "<td><a href=$form->{script}?path=$form->{path}&action=$action&type=$form->{type}&id=$ref->{id}&warehouse=$warehouse&vc=$form->{vc}&login=$form->{login}&callback=$callback>$ref->{$ordnumber}</a></td>";
 
     $name = $form->escape($ref->{name});
@@ -1869,7 +1896,7 @@ function CheckAll() {
 <br>
 |;
 
-  $form->hide_form(qw(callback type vc path login department ordnumber ponumber));
+  $form->hide_form(qw(callback type vc path login department ordnumber ponumber detail));
   
   print qq|
 
@@ -2144,7 +2171,8 @@ sub invoice {
     $form->{exchangerate} = $form->check_exchangerate(\%myconfig, $form->{currency}, $form->current_date(\%myconfig), $buysell);
   }
   
-  for (qw(id subject message cc bcc printed emailed queued)) { delete $form->{$_} }
+  for (qw(id subject message printed emailed queued)) { delete $form->{$_} }
+
   $form->{$form->{vc}} =~ s/--.*//g;
   $form->{type} = "invoice";
   $form->{formname} = "invoice";
@@ -2184,7 +2212,7 @@ sub invoice {
     for (qw(qty sellprice discount)) { $form->{"${_}_$i"} = $form->format_amount(\%myconfig, $form->{"${_}_$i"}) }
   }
 
-  for (qw(id subject message cc bcc printed emailed queued audittrail recurring)) { delete $form->{$_} }
+  for (qw(audittrail recurring)) { delete $form->{$_} }
 
   &display_form;
 
@@ -2234,7 +2262,7 @@ sub create_backorder {
     return;
   }
 
-  @flds = qw(partnumber description lineitemdetail qty ship unit sellprice discount oldqty oldship orderitems_id id bin weight listprice lastcost taxaccounts pricematrix sku onhand bin assembly inventory_accno_id income_accno_id expense_accno_id deliverydate reqdate itemnotes serialnumber projectnumber package netweight grossweight volume partsgroup);
+  @flds = qw(partnumber description lineitemdetail qty ship unit sellprice discount oldqty oldship orderitems_id id bin weight listprice lastcost taxaccounts pricematrix sku onhand bin assembly inventory_accno_id income_accno_id expense_accno_id deliverydate reqdate itemnotes serialnumber ordernumber customerponumber projectnumber package netweight grossweight volume partsgroup);
 
   for $i (1 .. $form->{rowcount}) {
     for (qw(qty sellprice discount)) { $form->{"${_}_$i"} = $form->format_amount(\%myconfig, $form->{"${_}_$i"}) }
@@ -2549,7 +2577,7 @@ sub display_ship_receive {
     print qq|
         </tr>
 |;
-    $form->hide_form(map { "${_}_$i"} qw(partnumber description sku orderitems_id id partsgroup unit bin qty));
+    $form->hide_form(map { "${_}_$i"} qw(partnumber description itemnotes lineitemdetail sku orderitems_id id partsgroup unit bin qty));
 
     if ($form->{type} eq 'ship_order') {
       print qq|
@@ -2662,6 +2690,260 @@ sub done {
 }
 
 
+sub search_transfer {
+  
+  OE->get_warehouses(\%myconfig, \%$form);
+
+  # warehouse
+  if (@{ $form->{all_warehouse} }) {
+    $form->{selectwarehouse} = "<option>\n";
+    $form->{warehouse} = qq|$form->{warehouse}--$form->{warehouse_id}| if $form->{warehouse_id};
+
+    for (@{ $form->{all_warehouse} }) { $form->{selectwarehouse} .= qq|<option value="$_->{description}--$_->{id}">$_->{description}\n| }
+  } else {
+    $form->error($locale->text('Nothing to transfer!'));
+  }
+  
+  $form->get_partsgroup(\%myconfig, { searchitems => 'part'});
+  if (@{ $form->{all_partsgroup} }) {
+    $form->{selectpartsgroup} = "<option>\n";
+    for (@{ $form->{all_partsgroup} }) { $form->{selectpartsgroup} .= qq|<option value="$_->{partsgroup}--$_->{id}">$_->{partsgroup}\n| }
+  }
+  
+  $form->{title} = $locale->text('Transfer Inventory');
+ 
+  $form->header;
+
+  print qq|
+<body>
+
+<form method=post action=$form->{script}>
+
+<table width=100%>
+  <tr>
+    <th class=listtop>$form->{title}</th>
+  </tr>
+  <tr height="5"></tr>
+  <tr>
+    <td>
+      <table>
+        <tr>
+          <th align=right nowrap>|.$locale->text('Transfer from').qq|</th>
+          <td><select name=fromwarehouse>$form->{selectwarehouse}</select></td>
+        </tr>
+        <tr>
+          <th align=right nowrap>|.$locale->text('Transfer to').qq|</th>
+          <td><select name=towarehouse>$form->{selectwarehouse}</select></td>
+        </tr>
+	<tr>
+	  <th align="right" nowrap="true">|.$locale->text('Part Number').qq|</th>
+	  <td><input name=partnumber size=20></td>
+	</tr>
+	<tr>
+	  <th align="right" nowrap="true">|.$locale->text('Description').qq|</th>
+	  <td><input name=description size=40></td>
+	</tr>
+	<tr>
+	  <th align=right nowrap>|.$locale->text('Group').qq|</th>
+	  <td><select name=partsgroup>$form->{selectpartsgroup}</select></td>
+	</tr>
+      </table>
+    </td>
+  </tr>
+  <tr>
+    <td><hr size=3 noshade></td>
+  </tr>
+</table>
+
+<br>
+<input type=hidden name=nextsub value=list_transfer>
+
+<input class=submit type=submit name=action value="|.$locale->text('Continue').qq|">|;
+
+  $form->hide_form(qw(path login sessionid));
+
+  print qq|
+</form>
+|;
+
+  if ($form->{menubar}) {
+    require "$form->{path}/menu.pl";
+    &menubar;
+  }
+
+  print qq|
+
+</body>
+</html>
+|;
+
+}
+
+
+sub list_transfer {
+
+  $form->{sort} = "partnumber" unless $form->{sort};
+
+  OE->get_inventory(\%myconfig, \%$form);
+  
+  # construct href
+  $href = "$form->{script}?action=list_transfer";
+  for (qw(direction oldsort path login sessionid)) { $href .= "&$_=$form->{$_}" }
+  for (qw(partnumber fromwarehouse towarehouse description partsgroup)) { $href .= "&$_=".$form->escape($form->{$_}) }
+
+  $form->sort_order();
+  
+  # construct callback
+  $callback = "$form->{script}?action=list_transfer";
+  for (qw(direction oldsort path login sessionid)) { $callback .= "&$_=$form->{$_}" }
+  for (qw(partnumber fromwarehouse towarehouse description partsgroup)) { $callback .= "&$_=".$form->escape($form->{$_},1) }
+
+  @column_index = $form->sort_columns(qw(partnumber description partsgroup make model fromwarehouse qty towarehouse transfer));
+
+  $column_header{partnumber} = qq|<th><a class=listheading href=$href&sort=partnumber>|.$locale->text('Part Number').qq|</a></th>|;
+  $column_header{description} = qq|<th><a class=listheading href=$href&sort=description>|.$locale->text('Description').qq|</a></th>|;
+  $column_header{partsgroup} = qq|<th><a class=listheading href=$href&sort=partsgroup>|.$locale->text('Group').qq|</a></th>|;
+  $column_header{fromwarehouse} = qq|<th><a class=listheading href=$href&sort=warehouse>|.$locale->text('From').qq|</a></th>|;
+  $column_header{towarehouse} = qq|<th class=listheading>|.$locale->text('To').qq|</th>|;
+  $column_header{qty} = qq|<th class=listheading>|.$locale->text('Qty').qq|</a></th>|;
+  $column_header{transfer} = qq|<th class=listheading>|.$locale->text('Transfer').qq|</a></th>|;
+
+  
+  ($warehouse, $warehouse_id) = split /--/, $form->{fromwarehouse};
+  
+  if ($form->{fromwarehouse}) {
+    $option .= "\n<br>";
+    $option .= $locale->text('From Warehouse')." : $warehouse";
+  }
+  ($warehouse, $warehouse_id) = split /--/, $form->{towarehouse};
+  if ($form->{towarehouse}) {
+    $option .= "\n<br>";
+    $option .= $locale->text('To Warehouse')." : $warehouse";
+  }
+  if ($form->{partnumber}) {
+    $option .= "\n<br>" if ($option);
+    $option .= $locale->text('Part Number')." : $form->{partnumber}";
+  }
+  if ($form->{description}) {
+    $option .= "\n<br>" if ($option);
+    $option .= $locale->text('Description')." : $form->{description}";
+  }
+  if ($form->{partsgroup}) {
+    ($partsgroup) = split /--/, $form->{partsgroup};
+    $option .= "\n<br>" if ($option);
+    $option .= $locale->text('Group')." : $partsgroup";
+  }
+
+  $form->{title} = $locale->text('Transfer Inventory');
+
+  $callback .= "&sort=$form->{sort}";
+  
+  $form->header;
+
+  print qq|
+<body>
+
+<form method=post action=$form->{script}>
+
+<input type=hidden name=warehouse_id value=$warehouse_id>
+
+<table width=100%>
+  <tr>
+    <th class=listtop>$form->{title}</th>
+  </tr>
+  <tr height="5"></tr>
+  <tr>
+    <td>$option</td>
+  </tr>
+  <tr>
+    <td>
+      <table width=100%>
+	<tr class=listheading>|;
+
+for (@column_index) { print "\n$column_header{$_}" }
+
+print qq|
+	</tr>
+|;
+
+  if (@{ $form->{all_inventory} }) {
+    $sameitem = $form->{all_inventory}->[0]->{$form->{sort}};
+  }
+
+  $i = 0;
+  foreach $ref (@{ $form->{all_inventory} }) {
+
+    $i++;
+
+    $column_data{partnumber} = qq|<td><input type=hidden name="id_$i" value=$ref->{id}>$ref->{partnumber}</td>|;
+    $column_data{description} = "<td>$ref->{description}&nbsp;</td>";
+    $column_data{partsgroup} = "<td>$ref->{partsgroup}&nbsp;</td>";
+    $column_data{fromwarehouse} = qq|<td><input type=hidden name="warehouse_id_$i" value=$ref->{warehouse_id}>$ref->{warehouse}&nbsp;</td>|;
+    $column_data{towarehouse} = qq|<td>$warehouse&nbsp;</td>|;
+    $column_data{qty} = qq|<td><input type=hidden name="qty_$i" value=$ref->{qty}>|.$form->format_amount(\%myconfig, $ref->{qty}).qq|</td>|;
+    $column_data{transfer} = qq|<td><input name="transfer_$i" size=4></td>|;
+
+    $j++; $j %= 2;
+    print "
+        <tr class=listrow$j>";
+    
+    for (@column_index) { print "\n$column_data{$_}" }
+
+    print qq|
+	</tr>
+|;
+
+  }
+  
+  print qq|
+      </table>
+    </td>
+  </tr>
+  
+  <tr>
+    <td><hr size=3 noshade></td>
+  </tr>
+</table>
+
+<br>
+
+<input name=callback type=hidden value="$callback">
+
+<input type=hidden name=rowcount value=$i>
+|;
+
+  $form->{action} = "transfer";
+  $form->hide_form(qw(path login sessionid action));
+
+  print qq|
+<input class=submit type=submit name=action value="|.$locale->text('Transfer').qq|">|;
+
+  if ($form->{menubar}) {
+    require "$form->{path}/menu.pl";
+    &menubar;
+  }
+
+  print qq|
+</form>
+
+</body>
+</html>
+|;
+
+
+}
+
+
+sub transfer {
+
+  if (OE->transfer(\%myconfig, \%$form)) {
+    $form->redirect($locale->text('Inventory transferred!'));
+  } else {
+    $form->error($locale->text('Could not transfer Inventory!'));
+  }
+
+}
+
 sub rfq_ { &add };
 sub quotation_ { &add };
 
@@ -2836,7 +3118,7 @@ BLANKROW:
 <br>
 |;
 
-  $form->hide_form(qw(callback department ponumber path login employee_id vc nextsub rowcount type));
+  $form->hide_form(qw(callback department ponumber path login employee_id vc nextsub rowcount type detail));
   
   print qq|
 <input class=submit type=submit name=action value="|.$locale->text('Generate Orders').qq|">|;
